--------------------------------------------------------------------------------
-- EETAC - CSD - P6 - http://digsys.upc.es
--------------------------------------------------------------------------------
-- Example of a HEX_to_7SEGMENTS decoder
-- Behavioural approach as described in:
-- http://digsys.upc.es/csd/P02/P2_T/P2_tut_BCD_7SEG_behav.html
--------------------------------------------------------------------------------

LIBRARY ieee;
USE  IEEE.STD_LOGIC_1164.all;

ENTITY HEX_7SEG_DECODER IS
		PORT (
			X_IN   		: IN STD_LOGIC_VECTOR(3 DOWNTO 0); 
			BI_L   		: IN STD_LOGIC;      
			segments  	: OUT STD_LOGIC_VECTOR(6 DOWNTO 0)
		);
END HEX_7SEG_DECODER;

ARCHITECTURE truth_table OF HEX_7SEG_DECODER IS

BEGIN
	PROCESS (BI_L, X_IN)  
		BEGIN
			IF BI_L = '1' THEN
				CASE X_IN IS  	--  Active-high outputs 
			       	   -- DCBA        abcdefg        
              		   -- ----        -------    
					WHEN "0000" =>       		-- 0
						segments <= "1111110";
					WHEN "0001" =>       		-- 1
						segments <= "0110000";
					WHEN "0010" =>       		-- 2
						segments <= "1101101";
					WHEN "0011" =>       		-- 3
						segments <= "1111001";
					WHEN "0100" =>       		-- 4
						segments <= "0110011";
					WHEN "0101" =>       		-- 5
						segments <= "1011011";
					WHEN "0110" =>       		-- 6
						segments <= "1011111";
					WHEN "0111" =>       		-- 7
						segments <= "1110000";
					WHEN "1000" =>       		-- 8
						segments <= "1111111";
					WHEN "1001" =>       		-- 9
						segments <= "1110011";
					WHEN "1010" =>       		-- A
						segments <= "1110111";
					WHEN "1011" =>       		-- b
						segments <= "0011111";
					WHEN "1100" =>       		-- C
						segments <= "1001110";
					WHEN "1101" =>       		-- d
						segments <= "0111101";
					WHEN "1110" =>       		-- E
						segments <= "1001111";
					WHEN Others =>       		-- F
						segments <= "1000111";
				END CASE;
			
			ELSE 
				segments <= "0000000"; --blank
			END IF;

	END PROCESS;
		
 	 		 		   	   
END truth_table;
