--------------------------------------------------------------------------------
-- Exercise P6  - CSD 
-- http://digsys.upc.es

-- Example of the implementation of the matrix_encoder_16key in the 
-- HWDLC4128V board (http://digsys.upc.es/ed/components/components.html#Lattice)
-- Hierarchical design using a top design and components

-- Main modules:
-- 16-key keypad scanner and decoder
-- HEX-7seg decoder
---------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;

ENTITY matrix_encoder_16key_top IS  
      PORT(	CD			: 	IN  std_logic;	-- (pin 17)
	  
           	OSC_CLK_in	:	IN std_logic;  -- from the FREQ_DIV (pin 89)
-- The board has a PLD working as a frequency divider: 1Hz, 10Hz, ... 1 MHz 
--         	OSC_CLK_in	:	IN std_logic;  -- from the timer 555 (pin 88)
-- The board has a 555 timer circuit to generate variable CLK frequencies
									-- (pin 81,79,80,85,87,84,86)
		   	segments	:	OUT std_logic_vector(6 downto 0); 
			
			-- The 7-seg display decimal point is GS
			GS			:	OUT std_logic; 	-- (pin 78)
								-- (pin 94,93,92,91) left connector				
			Rows 		:	OUT std_logic_vector(3 downto 0); 
								-- (pin 100,99,98,97) left connector				
			Columns		:	IN 	std_logic_vector(3 downto 0);
								-- (pin 61,60,59,58) LED's			
-- These other LEDs are used for debugging purposes, to inspect 
-- the HEX data and the row scan codes. The board contain 8 LEDs
			D_LEDs 		:	OUT	std_logic_vector(3 downto 0);
			R_LEDs 		:	OUT	std_logic_vector(3 downto 0)	
	);
END matrix_encoder_16key_top;

ARCHITECTURE schematic OF matrix_encoder_16key_top IS  

-- Components
	COMPONENT  HEX_7SEG_DECODER IS	-- display decoder 
	-- adapted form P2 tutorials
		PORT (
			X_IN		: IN 	STD_LOGIC_VECTOR(3 DOWNTO 0); 
			BI_L		: IN 	STD_LOGIC;    			
			segments	: OUT 	STD_LOGIC_VECTOR(6 downto 0)
		);
	END COMPONENT;

	COMPONENT matrix_encoder_16key IS  
		PORT(
			CLK, CD	:	IN  std_logic;			
			Columns	: 	IN  std_logic_vector (3 downto 0);
			Data	:	OUT std_logic_vector (3 downto 0);
			GS		:	OUT std_logic;
			Rows	:	OUT std_logic_vector (3 downto 0)	
          );
   END COMPONENT;

-- Signals to connect the components
	SIGNAL CLK			:	std_logic;
	SIGNAL Data			:	std_logic_vector (3 downto 0);
	SIGNAL Rows_Scan	:	std_logic_vector (3 downto 0);

	BEGIN  
-- Instantiation of components
	Chip2	: HEX_7SEG_DECODER
		PORT MAP (
		-- from component name 	=> to signal or port name
			X_IN		=> Data,
			segments	=> segments,
			BI_L		=> '1' -- Always enabled
		);

	Chip1	: matrix_encoder_16key
		PORT MAP (
		-- from component name 	=> to signal or port name
			CLK			=> CLK,
			CD			=> CD,
			Columns		=> Columns,
			Rows		=> Rows_Scan, 
			GS			=> GS,	
			Data		=> Data						
		);

 -- Connections and logic between components
	CLK 	<= OSC_CLK_in;
	D_LEDs 	<= Data;
	R_LEDs 	<= Rows_Scan;
	Rows 	<= Rows_Scan;	
END schematic ;
